@@ -6,10 +6,13 @@
 #include <sys/syscall.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <errno.h>
 #include <linux/unistd.h>
 #include <linux/raw.h>
 #include <linux/major.h>
 
+#include "indirect.h"
+
 #define FIO_HAVE_LIBAIO
 #define FIO_HAVE_POSIXAIO
 #define FIO_HAVE_FADVISE
@@ -23,18 +26,19 @@
 #define FIO_HAVE_HUGETLB
 #define FIO_HAVE_RAWBIND
 #define FIO_HAVE_BLKTRACE
+#define FIO_HAVE_STRSEP
+#define FIO_HAVE_FALLOCATE
+#define FIO_HAVE_POSIXAIO_FSYNC
+#define FIO_HAVE_PSHARED_MUTEX
 
-#define OS_MAP_ANON		(MAP_ANONYMOUS)
+#define OS_MAP_ANON		MAP_ANONYMOUS
 
 #ifndef CLOCK_MONOTONIC
 #define CLOCK_MONOTONIC 1
 #endif
 
-#ifdef FIO_HAVE_CPU_AFFINITY
 typedef cpu_set_t os_cpu_mask_t;
-#else
-typedef int os_cpu_mask_t;
-#endif
+
 typedef struct drand48_data os_random_state_t;
 
 /*
@@ -49,7 +53,6 @@ typedef struct drand48_data os_random_state_t;
  * If you are on an ancient glibc (2.3.2), then define GLIBC_2_3_2 if you want
  * the affinity helpers to work.
  */
-#ifdef FIO_HAVE_CPU_AFFINITY
 #ifndef GLIBC_2_3_2
 #define fio_setaffinity(td)		\
 	sched_setaffinity((td)->pid, sizeof((td)->o.cpumask), &(td)->o.cpumask)
@@ -61,7 +64,22 @@ typedef struct drand48_data os_random_state_t;
 #define fio_getaffinity(pid, ptr)	\
 	sched_getaffinity((pid), (ptr))
 #endif
-#endif
+
+#define fio_cpu_clear(mask, cpu)	CPU_CLR((cpu), (mask))
+#define fio_cpu_set(mask, cpu)		CPU_SET((cpu), (mask))
+
+static inline int fio_cpuset_init(os_cpu_mask_t *mask)
+{
+	CPU_ZERO(mask);
+	return 0;
+}
+
+static inline int fio_cpuset_exit(os_cpu_mask_t *mask)
+{
+	return 0;
+}
+
+#define FIO_MAX_CPUS			CPU_SETSIZE
 
 static inline int ioprio_set(int which, int who, int ioprio)
 {
@@ -111,7 +129,7 @@ struct async_head_user;
 static inline struct syslet_uatom *
 async_exec(struct syslet_uatom *atom, struct async_head_user *ahu)
 {
-	return (void *) syscall(__NR_async_exec, atom, ahu);
+	return (struct syslet_uatom *) syscall(__NR_async_exec, atom, ahu);
 }
 
 static inline long
@@ -225,4 +243,10 @@ static inline int fio_lookup_raw(dev_t dev, int *majdev, int *mindev)
 	return 0;
 }
 
+#ifdef O_NOATIME
+#define FIO_O_NOATIME	O_NOATIME
+#else
+#define FIO_O_NOATIME	0
+#endif
+
 #endif
